# sb::Plugin::AntiSpam
# == written by T.Otani <takuya.otani@gmail.com> ===
# == Copyright (C) 2004 SimpleBoxes/SerendipityNZ ==

use 5.006;
use strict;

if ( !defined $sb::VERSION
  or $sb::VERSION =~ /^2.(\d\d)[BRD]$/i )
{
	die("This plugin works under Serene Bach 3.* only.\n");
}

# ======================================
package sb::Plugin::AntiSpam;
use sb::Plugin ();
# --------------------------------------
sb::Plugin->register_plugin(
	'lang' => {
		'ja' => 'utf8',
		'en' => 'ascii',
	},
	'text' => {
		'type'    => 'admin, receipt',
		'name'    => 'AntiSpam',
		'text'    => 'AntiSpam plugin for Serene Bach',
		'author'  => 'takkyun',
		'detail'  => 'http://serenebach.net/',
		'version' => version(),
	},
	'file' => 'antispam.txt',
	'data' => 1,
);
sb::Plugin->register_admin_module(
	'mode'   => 'antispam',
	'level'  => 1,
	'module' => 'sb::Admin::AntiSpam',
);
sb::Plugin->register_extra_module(
	'type'     => 'receipt',
	'callback' => \&sb::App::Receipt::_check_via_antispam,
);
sub version
{
	return '0.06';
	# 0.06 [2008/12/02] modified Net::Akismet to use sb::Net instead sb::App, renamed sb::App::Receipt from sb::Receipt
	# 0.05 [2008/11/24] fixed typo
	# 0.04 [2008/11/12] modified for Serene Bach 3.*
	# 0.03 [2008/05/30] renamed as "AntiSpam" due to adding TypePad service
	# 0.02 [2007/02/21] fixed a bug in _save_data
	# 0.01 [2007/02/16] added trackback checking feature
	# 0.00 [2007/02/15] generated
}
# ======================================
package sb::Admin::AntiSpam;
use sb::Text ();
use sb::App::Admin ();
use base qw( sb::App::Admin );
# --------------------------------------
# // defines
sub TEMPLATE (){ 'antispam.html' };
# --------------------------------------
# // public functions
sub callback
{
	my $self = shift;
	return ( $self->regi() ) 
		? $self->_update(@_)
		: $self->_display(@_);
}
# --------------------------------------
# // private functions
sub _update
{
	my $self = shift;
	my %param = ( 'message' => '', @_ );
	my $data = sb::Plugin->get_data;
	$data->data(sb::Text->entitize($self->cgi->value('akismet_apikey')));
	$data->text(sb::Text->entitize($self->cgi->value('typepad_apikey')));
	$data->setting(($self->cgi->value('antispam_tb') eq 'yes') ? 1 : 0);
	sb::Plugin->set_data('data'=>$data);
	return $self->_display('message'=>sb->string('Modified.<br />'));
}
sub _display
{
	my $self = shift;
	my %param = ( 'message' => '', @_ );
	my $temp = sb::Template->new(sb::Plugin->load_template('file'=>TEMPLATE));
	my $data = sb::Plugin->get_data;
	$temp->num(0);
	$temp->tag('akismet_apikey' => $data->data);
	$temp->tag('typepad_apikey' => $data->text);
	$temp->tag('antispam_tb' => 'checked="checked"') if ($data->setting == 1);
	$self->common_parts($temp);
	$self->set_message($temp,$param{'message'});
	return $self->set_main($temp->output);
}
# ======================================
package sb::App::Receipt;
sub _get_server
{
	my $name = shift;
	return 'api.antispam.typepad.com' if ($name eq 'typepad');
	return 'rest.akismet.com';
}
sub _check_via_antispam
{
	my $self  = shift;
	my $entry = shift; # entry object : sb::Data::Entry
	my $type  = shift; # 'tb' or 'com'
	my $obj   = shift; # data hash reference (not sb::Data::Object)
	my $data = sb::Plugin->get_data;
	return 0 if (!$data); # do nothing
	return 0 if ($type eq 'tb' and $data->setting != 1);
	my %key_of = (
		'akismet' => $data->data,
		'typepad' => $data->text,
	);
	foreach my $name ( keys %key_of )
	{
		next if ($key_of{$name} eq ''); # key is empty, so skip it
		my $client = Net::Akismet->new(
			'KEY'    => $key_of{$name},
			'URL'    => $self->conf->value('conf_srv_base'),
			'SERVER' => $self->_get_server($name),
		);
		next if (!$client); # failed to create a client
		my $verdict = undef;
		if ($type eq 'tb')
		{ # for trackback
			$verdict =  $client->check(
				'USER_IP'              => $obj->{'host'},
				'COMMENT_TYPE'         => 'trackback',
				'COMMENT_CONTENT'      => $obj->{'body'},
				'COMMENT_AUTHOR'       => $obj->{'name'},
				'COMMENT_AUTHOR_URL'   => $obj->{'url'},
				'PERMALINK'            => $entry->permalink,
			);
		}
		else
		{ # for comment
			$verdict =  $client->check(
				'USER_IP'              => $obj->{'host'},
				'COMMENT_TYPE'         => 'comment',
				'COMMENT_CONTENT'      => $obj->{'body'},
				'COMMENT_AUTHOR'       => $obj->{'auth'},
				'COMMENT_AUTHOR_EMAIL' => $obj->{'mail'},
				'COMMENT_AUTHOR_URL'   => $obj->{'url'},
				'REFERRER'             => $self->cgi->value('_refe'),
				'PERMALINK'            => $entry->permalink,
			);
		}
		return 1 if ($verdict eq 'true');
	}
	return 0;
}
# ======================================
# the following package Net::Akismet is based on Net::Akismet 0.02.
# --------------------------------------
# Copyright (C) 2006 by Nikolay Bachiyski - Net::Akismet 0.02
# This library is free software; you can redistribute it and/or modify
# it under the same terms as Perl itself, either Perl version 5.8.7 or,
# at your option, any later version of Perl 5 you may have available.
package Net::Akismet;
# --------------------------------------
# // configuration for inheritance / dependancy
use sb::Net ();
use HTTP::Request::Common;
# --------------------------------------
# // declaration for constant value
sub SERVER     (){ 'rest.akismet.com' };
sub VERSION    (){ '1.1' };
sub LIBVERSION (){ 0.03 };
# --------------------------------------
# // public functions
sub new
{
	my $that = shift;
	my $class = ref $that || $that;
	my %params = @_;
	my $self = \%params;
	$self->{ua} = sb::Net->new->agent() or return undef;
	my $key = $self->{KEY} or return undef;
	my $url = $self->{URL} or return undef;
	$self->{SERVER}      = SERVER  if ($self->{SERVER} eq '');      # default server
	$self->{API_VERSION} = VERSION if ($self->{API_VERSION} eq ''); # default api version
	$self->{ua}->agent($sb::PRODUCT . '/' . $sb::VERSION . ' | Akismet Perl/' . LIBVERSION );
	bless $self, $class;
	return $self->_verify_key() ? $self : undef;
}
sub check
{
	my $self = shift;
	$self->_submit('comment-check', {@_}) or return undef;
	('true' eq $self->{response} || 'false' eq $self->{response}) or return undef;
	return $self->{response};
}
sub spam
{
	my $self = shift;
	return $self->_submit('submit-spam', {@_});
}
sub ham
{
	my $self = shift;
	return $self->_submit('submit-ham', {@_});
}
# --------------------------------------
# // private functions
sub _verify_key
{
	my $self = shift;
	my $response = $self->{ua}->request(
		POST 'http://' . $self->{SERVER} . '/' . $self->{API_VERSION} . '/verify-key', 
		[
			key  => $self->{KEY},
			blog => $self->{URL},
		]
	);
	($response && $response->is_success() && 'valid' eq $response->content()) or return undef;
	return 1;
}
sub _submit
{
	my $self = shift;
	my $action = shift || 'comment-check';
	my $comment = shift;
	$comment->{USER_IP} || return undef;
	my $response = $self->{ua}->request(
		POST 'http://' . $self->{KEY} . '.' . $self->{SERVER} . '/' . $self->{API_VERSION} . '/' . $action,
		[
			blog                 => $self->{URL},
			user_ip              => $comment->{USER_IP},
			user_agent           => $self->{ua}->agent(),
			referrer             => $comment->{REFERRER},
			permalink            => $comment->{PERMALINK},
			comment_type         => $comment->{COMMENT_TYPE},
			comment_author       => $comment->{COMMENT_AUTHOR},
			comment_author_email => $comment->{COMMENT_AUTHOR_EMAIL},
			comment_author_url   => $comment->{COMMENT_AUTHOR_URL},
			comment_content      => $comment->{COMMENT_CONTENT},
		]
	);
	($response && $response->is_success()) or return undef;
	$self->{response} = $response->content();
	return 1;
}
1;
__END__
